/* This program gives the estimated positions for the 4 major Jupiter moons.
    It is meant to use the Serial Monitor as interaction, but also uses a NeoPixel
    string to display the results of calculations. The program starts by asking for
    desired date and time. Make sure in the Serial Monitor that you have choosen
    'Newline' in the dropdown box on the bottom of the window. Enter the year as a
    4 digit integer. Enter the month as a 1 or 2 digit integer number. Enter the
    day as a decimal number adjusted to universal time (add or subtract your time zone
    to get UT - the time at the prime meridian). Convert your local time to a decimal
    day by dividing by 24. For instance, 6:00am on 5 March UT would be a decimal day of
    5.25. The result is the calculation of the distance from Jupiter in Jupiter radii of
    the 4 moons. It then shows a graphical representation of the layout to help you
    better visualize the orientation. The program also assumes you are looking through
    a telescope, so it will appear backwards to those viewing with binoculars or the
    naked eye. Visit subsystems.us for more fun Arduino and science projets.
*/

#define PIX_COUNT 60 // Max number of Neo Pixels
#define PIX_PIN    6 // Pin that communicates with NeoPixels
#include <Adafruit_NeoPixel.h>
#include <TimeLib.h>

Adafruit_NeoPixel strip(PIX_COUNT, PIX_PIN, NEO_GRB + NEO_KHZ800);

int dYear = 2020; // Put the current year here

int dMonth = 7; // Put current month here

//**** Enter time in UTC ****//
int tDay = 6; // Put current Day here
int tHour = 12; // Put current Hour here
int tMin = 0; // Put current minute here

float dDay = tDay + tHour / 24 + tMin / 1440; // calculate fractional day

void setup() {

  setTime(tHour, tMin, 0, tDay, dMonth, dYear);

  strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.show();            // Turn OFF all pixels ASAP
  strip.setBrightness(40); // Set BRIGHTNESS to about 1/5 (max = 255)
}

// Routine to calculate the position of Jupiter and its moons

void jSats(int tYear, int tMonth, float tDay) {
  int cYear, cMonth;
  if (tMonth < 3) {
    cYear = tYear - 1;
    cMonth = tMonth + 12;
  } else {
    cYear = tYear;
    cMonth = tMonth;
  }
  // Calculate the Julian Date offset from Epoch
  int a = cYear / 100;
  int b = 2 - a + (int)(a / 4);
  long c = 365.25 * cYear;
  long d = 30.6001 * (cMonth + 1);
  float N = b + c + d + tDay - 694025.5;

  // Calc moon positions
  float P = PI / 180.0;
  float MT = (358.476 + 0.9856003 * N) * P;
  float MJ = (225.328 + 0.0830853 * N) * P;
  float JJ = 221.647 + 0.9025179 * N;
  float VT = 1.92 * sin(MT) + 0.02 * sin(2 * MT);
  float VJ = 5.55 * sin(MJ) + 0.17 * sin(2 * MJ);
  float K = (JJ + VT - VJ) * P;
  float DT = sqrt(28.07 - 10.406 * cos(K));
  float Z1 = sin(K) / DT;
  float I = atan(Z1 / sqrt(1 - Z1 * Z1));
  I = I / P;
  float F = (N - DT / 173);
  float F1 = I - VJ;
  float U1 = 84.5506 + 203.405863 * F + F1;
  float U2 = 41.5015 + 101.2916323 * F + F1;
  float U3 = 109.9770 + 50.2345169 * F + F1;
  float U4 = 176.3586 + 21.4879802 * F + F1;
  float X1 = 5.906 * sin(U1 * P + PI); // Distance of IO
  float X2 = 9.397 * sin(U2 * P + PI); // Distance of Europa
  float X3 = 14.989 * sin(U3 * P + PI); // Distance of Ganymede
  float X4 = 26.364 * sin(U4 * P + PI); // Distance of Callisto

  // Show the lights to correspond to the moons

  strip.clear();
  int pos = X1 + 30;

  strip.setPixelColor(pos, strip.Color(  0, 255,   0));
  pos = X2 + 30;
  
  strip.setPixelColor(pos, strip.Color(  0, 0,   255));
  pos = X3 + 30;

  strip.setPixelColor(pos, strip.Color(  255, 0,   255));
  pos = X4 + 30;

  strip.setPixelColor(pos, strip.Color(  255, 255,   0));

  strip.setPixelColor(30, strip.Color(  255, 0,   0));
  strip.show();
}



void loop() {
  jSats(year(), month(), (float)day() + (float)hour() / 24 + (float)minute() / 1440);

// Uncomment the below line to watch an annimation of the planets as
// this will add 2 minutes to the time everytime the loop executes.

  adjustTime(120);
}
